require "httparty"
require "json"

class VexillaClient

  @@VEXILLA_FEATURE_TYPE_TOGGLE = "toggle"
  @@VEXILLA_FEATURE_TYPE_GRADUAL = "gradual"

  def initialize(environment, base_url, custom_instance_hash)
    @environment = environment
    @base_url = base_url
    @custom_instance_hash = custom_instance_hash
  end

  def fetch_flags(file_name)
    response = HTTParty.get("#{@base_url}/#{file_name}")
    parsed_response = JSON.parse(response.body)
    parsed_response["environments"]
  end

  def set_flags(flags)
    @flags = flags
  end

  def should(feature_name)
    feature = @flags[@environment]["untagged"][feature_name]

    if feature["type"] == @@VEXILLA_FEATURE_TYPE_TOGGLE then
      feature["value"]
    elsif feature["type"] == @@VEXILLA_FEATURE_TYPE_GRADUAL
      hash_value = hash_instance_id(feature["seed"])
      hash_value <= feature["value"]
    else
      false
    end
  end

  private

  def hash_instance_id(seed)
    chars = @custom_instance_hash.chars

    total = chars.map { |char| char.ord }.reduce(0) { |sum, char_code| sum + char_code }

    (total * seed * 42.0) % 100
  end

end
